#include "grid.h"
using namespace std;
Grid::Grid(QWidget *parent,unsigned int w,unsigned int h) :
	QWidget(parent)
{
	setCellColor(1,(palette().brush(QPalette::Dark)).color());
	setCellColor(3,(palette().brush(QPalette::Highlight)).color());
	gridState=0;
	setAutoFillBackground(true);
	setBackgroundRole(QPalette::Base);
	setGridSize(w,h);
}
void Grid::paintEvent(QPaintEvent *e){
	QPainter p;
	p.begin(this);
	paintGrid(p);
	paintGridState(p);
	p.end();
	QWidget::paintEvent(e);
}

void Grid::setAutoSet(bool enable){
	autoSet=enable;
}

bool Grid::getAutoSet(){
	return autoSet;
}

QSize Grid::cellSize() const{
	return QSize(width()/gridWidth,height()/gridHeight);
}

Grid::~Grid(){
	deleteGrid();
}

void Grid::paintGrid(QPainter &p){
	int cellWidth=cellSize().width();
	int cellHeight=cellSize().height();
	int w=gridWidth*cellWidth;
	int h=gridHeight*cellHeight;
	for(unsigned int i=0;i<=gridWidth;i++)
		p.drawLine(i*cellWidth,0,i*cellWidth,h);
	for(unsigned int i=0;i<=gridHeight;i++)
		p.drawLine(0,i*cellHeight,w,i*cellHeight);
}

void Grid::paintGridState(QPainter &p){
	int cellWidth=cellSize().width();
	int cellHeight=cellSize().height();
	int w=p.pen().width();
	w=w?w:1;
	for(unsigned int i=0;i<gridWidth;i++)
		for(unsigned int i2=0;i2<gridHeight;i2++)
			if(getCell(i,i2)){
				QColor c=colors[0];
				if(colors.count()>getCell(i,i2))
					c=colors[getCell(i,i2)];
				p.fillRect(i*cellWidth+w,i2*cellHeight+w,cellWidth-w,cellHeight-w,QBrush(c));
			}
}

void Grid::clearGrid(){
	for(unsigned int i=0;i<gridWidth;i++)
		for(unsigned int i2=0;i2<gridHeight;i2++)
			clearCell(i,i2);
}

void Grid::clearCells(char state){
	for(unsigned int i=0;i<gridWidth;i++)
		for(unsigned int i2=0;i2<gridHeight;i2++)
			if(getCell(i,i2)==state)clearCell(i,i2);
}

const char *const * Grid::getGrid() const{
	return gridState;
}

void Grid::setGridSize(unsigned int width, unsigned int height, bool preserve){
	char** tmp=new char* [width];
	for(unsigned int i=0;i<width;i++){
		tmp[i]=new char[height];
		memset(tmp[i],0,height);
	}

	if(preserve&&gridState)
		for(unsigned int i=0;i<min(this->gridWidth,width);i++)
			for(unsigned int i2=0;i2<min(this->gridHeight,height);i2++)
				tmp[i][i2]=gridState[i][i2];

	deleteGrid();

	this->gridWidth=width;
	this->gridHeight=height;
	gridState=tmp;
	update();
}

void Grid::deleteGrid(){
	if(!gridState)return;
	for(unsigned int i=0;i<gridWidth;i++)
		delete[] gridState[i];
	gridState=0;
	gridWidth=0;
	gridHeight=0;
}

void Grid::setGrid(const char *const * grid, unsigned int width, unsigned int height){
	deleteGrid();
	gridState=new char* [width];
	for(unsigned int i=0;i<width;i++)
		gridState[i]=new char[height];

	for(unsigned int i=0;i<width;i++)
		for(unsigned int i2=0;i2<height;i2++)
			gridState[i][i2]=grid[i][i2];
}

bool Grid::setCell(unsigned int x,unsigned int y, char state){
	if((x>=gridWidth)||(y>=gridHeight))
		return false;
	gridState[x][y]=state;
	update();
	return true;
}

bool Grid::clearCell (unsigned int x, unsigned int y){
	return setCell(x,y,0);
}

const QSize Grid::getGridSize()const{
	return QSize(gridWidth,gridHeight);
}

void Grid::mouseReleaseEvent(QMouseEvent * m){
	QPoint p=decodePosition(m->pos());
	if(p.x()>=(int)gridWidth||p.y()>=(int)gridHeight)return;
	if(autoSet){
		switch (m->button()){
			case Qt::MidButton:
				setCell(p,2);
				break;
			case Qt::RightButton:
				clearCell(p);
				break;
			case Qt::LeftButton:
			default:
				setCell(p);
				break;
		}
	}
	emit clicked(p,m->button());
}

bool Grid::setCell(QPoint p, char state){
	return setCell(p.x(),p.y(),state);
}

bool Grid::clearCell (QPoint p){
	return clearCell(p.x(),p.y());
}

QPoint Grid::decodePosition(QPoint p) const{
	return QPoint(p.x()/cellSize().width(),p.y()/cellSize().height());
}

char Grid::getCell(QPoint p) const{
	return getCell(p.x(),p.y());
}

char Grid::getCell(unsigned int x, unsigned int y) const{
	if (x>=gridWidth||y>=gridHeight) return 0;
	return gridState[x][y];
}

void Grid::setCellColor(char state, QColor color){
	while(colors.count()<=state){
		colors.append((palette().brush(QPalette::Button)).color());
	}
	colors[state]=color;
}
