/*!\file messaging.h
 * \author F.Satzger
*/

#ifndef MESSAGING_H
#define MESSAGING_H

#include <QString>
#include <QIcon>
#include <QObject>

/*! \brief Gibt Nachrichten, Warnungen und Fehler an den Benutzer weiter.
 *
 * Muss mit einem entsprechenden Slot zur Ausgabe gekoppelt werden.\n
 * Benutzungsbeispiel:\n
 * Messaging::message("Programm gestartet.");
 */
class Messaging:public QObject
{
		Q_OBJECT
	public:
		//! \return Ein Messaging-Objekt gemäß Singleton-Pattern
		static Messaging* getObject();
		//! \brief Erzeugt eine Info-Nachricht
		static void message(QString msg);
		//! \brief Erzeugt eine Fehlermeldung
		static void error (QString err);
		//! \brief Erzeugt eine Warnung (~ nicht-kritischer Fehler)
		static void warning (QString warn);

		//! Setzt den Fortschritt durch emittieren eines entsprechenden Signals
		static void setProgress(double progress);

	signals:
		//! \param msg Nachricht
		//! \param icon Symbol zur Nachricht
		void addMessage(QString msg,QIcon icon);

		//! Wird emittiert, wenn setProgress() aufgerufen wird.
		void progress(double p);

	private:

		static Messaging* instance;
		Messaging(QObject* parent=0);
};

#endif // MESSAGING_H
