#ifndef SETTINGS_H
#define SETTINGS_H

#include <QSettings>
#include <QDir>
#include <QFile>

//! Enthält alle Einstellungen des Programms (Singleton).
class Settings : public QSettings
{
		Q_OBJECT
	public:
		/*! Lädt ein anderes Present
		 * \attention Viele Widgets laden neue Einstellungen nur im Construktor! Ein Neustart
		 * des Programms könnte erforderlich sein.
		 */
		static void loadSettings(QString present="Vorherige");
		//! Kopiert ein Present
		static bool copySettings(QString existing, QString neu);
		//! Gibt eine Liste aller existierenden Presents zurück
		static QStringList getPresents();
		/*! Gibt die aktuellen Settings zurück.
		 * die Settings können zuvor mit loadSettings() gewählt werden.
		 */
		static Settings& getInstance();
		//! Gibt den Pfad zu ImageMagick-convert zurück
		QString getConvert();
		//! Gibt den Pfad zu ImageMagick-mogrify zurück
		QString getMogrify();
		//! Gibt den Pfad zu ImageMagick-composite zurück
		QString getComposite();
		//! Setzt den Pfad zu ImageMagick-convert
		void setComposite(QString path);
		//! Setzt den Pfad zu ImageMagick-mogrify
		void setMogrify(QString path);
		//! Setzt den Pfad zu ImageMagick-composite
		void setConvert(QString path);
		//! Der Name des aktuell geladenen Presents
		const QString present;
		/*! Speichert die aktuellen Einstellungen unter dem gegebenen Namen
		 * \param present Name des neuen Presents
		 */
		void savePresent(QString present);
	signals:

	public slots:

	private:
		Settings(QObject *parent,QString present);
		static Settings *instance;


};

#endif // SETTINGS_H
