#include "startposition.h"


startposition::startposition(QWidget *parent) :
	ProcessingWidget(parent),startPositionState(-1),bsave(trUtf8("Speichern")),
	bsavePresent(trUtf8("Einstellungen speichern"))
{
	grid.setCellColor(1,QColor(0,0,255));	//überflutet
	grid.setCellColor(2,QColor(0,180,255)); //nass
	grid.setCellColor(3,QColor(215,128,0)); //trocken
	grid.setCellColor(4,Qt::green);			//Startposition
	grid.setAutoSet(false);
	layout.addWidget(&grid,0,0,3,1);
	layout.addWidget(&bsave,0,1);
	layout.addWidget(&bsavePresent,1,1);
	layout.setColumnStretch(0,1);
	setLayout(&layout);

	soriginal=Workdata::getTempFileName()+".png";
	shell=Workdata::getTempFileName()+".png";
	sdunkel=Workdata::getTempFileName()+".png";
	smedium=Workdata::getTempFileName()+".png";
	smask1=Workdata::getTempFileName()+".png";
	smask2=Workdata::getTempFileName()+".png";
	smap=Workdata::getTempFileName();
	connect(&grid,SIGNAL(clicked(QPoint,Qt::MouseButton)),
			SLOT(startPositionSelected(QPoint,Qt::MouseButton)));
	connect(&bsave,SIGNAL(clicked()),SLOT(save()));
	connect(&bsavePresent,SIGNAL(clicked()),SLOT(savePresent()));
}

void startposition::savePresent(){
	QString present=QInputDialog::getText(this,trUtf8("Present speichern"),trUtf8("Bitte Name des Presents angeben. Vorhandene Presents werden überschrieben"),QLineEdit::Normal,Settings::getInstance().present);
	if(present.isEmpty())return;
	present.replace(" ","_");
	Settings::getInstance().savePresent(present);
	Settings::loadSettings(present);
}

void startposition::save(){
	if(startPositionState<0){
		Messaging::warning(trUtf8("Startposition automatisch gesetzt!"));
		for(int x=0;x<grid.getGridSize().width();x++)
			for(int y=0;y<grid.getGridSize().height();y++)
				if(grid.getCell(x,y)!=1){
					startPositionSelected(QPoint(x,y),Qt::LeftButton);
					goto weiter;
				}

	}
	weiter:
	QString fn=QFileDialog::getSaveFileName(this,trUtf8("Karte speichern..."));
	if(fn=="")return;
	if(QFile::exists(fn))QFile::remove(fn);
	QFile::copy(smap,fn);
	QFile f(fn);
	if(!f.open(QFile::Append)){
		Messaging::error(trUtf8("Konnte Datei nicht öffnen!"));
		return;
	}
	QString startpos="ROUND 1 "+QString::number(startPosition.x()+1)+","+
			QString::number(startPosition.y()+1)+"\n";
	f.write(startpos.toLocal8Bit().data());
	f.close();
	Messaging::message(trUtf8("Karte gespeichert unter ")+fn);
}

bool startposition::activate(){
	Messaging::message("Erstelle Kartenvorschau...");
	Messaging::setProgress(0);

	Workdata::convert("-resize "+QString::number(Settings::getInstance().value("map/width").toInt())
					+" -alpha off",Workdata::getInstance().getFilename("original"),soriginal);
	Messaging::setProgress(0.1);
	switch(Settings::getInstance().value("vorauswahl").toInt()){
		case 0:
			Workdata::mogrify("-colorspace Gray",soriginal);
			break;
		case 1:
			Workdata::mogrify("-channel R -separate",soriginal);
			break;
		case 2:
			Workdata::mogrify("-channel B -separate",soriginal);
			break;
		case 3:
			Workdata::mogrify("-channel G -separate",soriginal);
			break;
	}
	Messaging::setProgress(0.2);
	Workdata::convert("-brightness-contrast "+QString::number(-Settings::getInstance().value
					("dunkel/value").toInt())+"x100",soriginal,sdunkel);
	Workdata::convert("-negate -brightness-contrast "+QString::number(-Settings::getInstance().
					value("hell/value").toInt())+"x100",soriginal,shell);
	Messaging::setProgress(0.3);
	Workdata::composite(sdunkel,shell,shell,smedium);
	if(QFile::exists(smask1))QFile::remove(smask1);
	QFile::copy(shell,smask1);
	Workdata::mogrify("-negate",shell);
	Messaging::setProgress(0.4);
	Workdata::mogrify("-negate",smedium);
	Messaging::setProgress(0.5);
	Workdata::composite(smedium,smask1,smask1,smask2);
	Messaging::setProgress(0.6);
	Workdata::mogrify("-brightness-contrast 50",smedium);
	Messaging::setProgress(0.7);
	Workdata::composite(smedium,shell,smask1,soriginal);
	Workdata::composite(sdunkel,soriginal,smask2,soriginal);
	Messaging::setProgress(0.8);
	png2field(soriginal.toLocal8Bit().data(),smap.toLocal8Bit().data());
	Messaging::setProgress(0.9);

	QFile f(smap);
	if (!f.open(QIODevice::ReadOnly | QIODevice::Text)){
		Messaging::error(trUtf8("Fehler beim einlesen der Kartenvorschau"));
		return false;
	}

	QTextStream text(&f);
	QStringList start=text.readLine().split(" "); //GAMEBOARDSTART x,y
	start=start[1].split(",");
	grid.setGridSize(start[0].toInt(),start[1].toInt());

	for(int y=0;y<grid.getGridSize().height();y++){
		QString line=text.readLine();
		for(unsigned int x=0;x<(unsigned int)grid.getGridSize().width();x++){
			switch (line[x].toLatin1()){
				case '.':
					grid.setCell(x,y,1);
					break;
				case 'o':
					grid.setCell(x,y,2);
					break;
				case '#':
					grid.setCell(x,y,3);
					break;
				default:
					Messaging::error(trUtf8("Fehler im Spielfeld."));
					return false;
			}
		}
	}


	f.close();
	Messaging::setProgress(1);
	Messaging::message("Kartenvorschau erstellt.");
	return true;
}

startposition::~startposition(){
	QFile::remove(shell);
	QFile::remove(soriginal);
	QFile::remove(sdunkel);
	QFile::remove(smedium);
	QFile::remove(smask1);
	QFile::remove(smask2);
	QFile::remove(smap);

}

QString startposition::information(){
	return trUtf8("Bitte Startposition wählen und fertige Karte abspeichern. \n\nDie Einstellungen können \
zur Weiterverwendung mit einem Namen gespeichert werden.");
}

void startposition::startPositionSelected(QPoint cell, Qt::MouseButton){
	if(grid.getCell(cell)==1){
		QMessageBox::warning(this,trUtf8("Fehler"),trUtf8("Der Startpunkt darf nicht auf einem \
überfluteten Feld liegen!"));
		return;
	}
	if(cell==startPosition)return;
	if(startPositionState>0){
		grid.setCell(startPosition,startPositionState);
	}
	startPositionState=grid.getCell(cell);
	grid.setCell(cell,4);
	startPosition=cell;
}
