#include "startseite.h"

startseite::startseite(QWidget *parent) :
	ProcessingWidget(parent),expertMode(trUtf8("erweiterte Einstellungen")),lfilename(trUtf8("Bild:")),
	lsize(trUtf8("Spielfeldgröße:")),lsize2("x"),lmogrify(trUtf8("Mogrify:")),lconvert(trUtf8("Convert:")),
	lcomposite(trUtf8("Composite:")),lpresent(trUtf8("Voreinstellung:")),
	bfilename(trUtf8("...")),bmogrify(trUtf8("...")),bconvert(trUtf8("...")),bcomposite(trUtf8("..."))
{
	setLayout(&layout);
	layout.addWidget(&lmogrify,0,0);
	layout.addWidget(&emogrify,0,1);
	layout.addWidget(&bmogrify,0,2,1,2);
	layout.addWidget(&lconvert,1,0);
	layout.addWidget(&econvert,1,1);
	layout.addWidget(&bconvert,1,2,1,2);
	layout.addWidget(&lcomposite,2,0);
	layout.addWidget(&ecomposite,2,1);
	layout.addWidget(&bcomposite,2,2,1,2);

	layout.addWidget(&lfilename,3,0);
	layout.addWidget(&efilename,3,1);
	layout.addWidget(&bfilename,3,2,1,2);

	layout.addWidget(&lsize,4,0);
	layout.addWidget(&ewidth,4,1);
	layout.addWidget(&lsize2,4,2);
	layout.addWidget(&eheight,4,3);

	layout.addWidget(&expertMode,5,0,1,2);
	layout.addWidget(&lpresent,5,2);
	layout.addWidget(&cpresent,5,3);

	connect(&bfilename,SIGNAL(clicked()),SLOT(selectFile()));
	connect(&bconvert,SIGNAL(clicked()),SLOT(selectConvert()));
	connect(&bmogrify,SIGNAL(clicked()),SLOT(selectMogrify()));
	connect(&bcomposite,SIGNAL(clicked()),SLOT(selectComposite()));
	connect(&expertMode,SIGNAL(clicked(bool)),SLOT(checkExpert(bool)));
	connect(&efilename,SIGNAL(editingFinished()),SLOT(loadFile()));
	connect(&ewidth,SIGNAL(editingFinished()),SLOT(widthChanged()));
	connect(&eheight,SIGNAL(editingFinished()),SLOT(heightChanged()));
	connect(&emogrify,SIGNAL(editingFinished()),SLOT(mogrifyChanged()));
	connect(&econvert,SIGNAL(editingFinished()),SLOT(convertChanged()));
	connect(&ecomposite,SIGNAL(editingFinished()),SLOT(compositeChanged()));
	connect(&cpresent,SIGNAL(currentIndexChanged(QString)),SLOT(presentChanged(QString)));

}

void startseite::presentChanged(QString newPresent){
	if(activating)return;
	Settings::loadSettings(newPresent);
	activate();
}

bool startseite::activate(){
	activating=true;
	ecomposite.setText(Settings::getInstance().getComposite());
	econvert.setText(Settings::getInstance().getConvert());
	emogrify.setText(Settings::getInstance().getMogrify());
	eheight.setText(Settings::getInstance().value("map/height").toString());
	ewidth.setText(Settings::getInstance().value("map/width",100).toString());
	cpresent.clear();
	foreach(QString present, Settings::getPresents()){
		cpresent.addItem(present);
	}
	cpresent.setEditable(false);
	cpresent.setCurrentIndex(cpresent.findText(Settings::getInstance().present));
	activating=false;
	return true;
}

void startseite::checkExpert(bool checked){
	emit enableExpert(checked);
}

void startseite::selectFile(){
	QString fn=QFileDialog::getOpenFileName(0,trUtf8("Bild auswählen"),"", trUtf8("Bilder (*.png *.jpg *.bmp)"));
	if(fn=="")return;
	efilename.setText(fn);
	loadFile();
}

void startseite::selectConvert(){
	QString fn=QFileDialog::getOpenFileName(0,trUtf8("Convert-Binary auswählen"),"");
	if(fn=="")return;
	econvert.setText(fn);
	convertChanged();
}

void startseite::selectMogrify(){
	QString fn=QFileDialog::getOpenFileName(0,trUtf8("Mogrify-Binary auswählen"),"");
	if(fn=="")return;
	emogrify.setText(fn);
	mogrifyChanged();
}


void startseite::selectComposite(){
	QString fn=QFileDialog::getOpenFileName(0,trUtf8("Composite-Binary auswählen"),"");
	if(fn=="")return;
	ecomposite.setText(fn);
	compositeChanged();
}

void startseite::widthChanged(){
	if(!Workdata::isInitalized())return;
	eheight.setText(QString::number(ewidth.text().toInt()*Workdata::getInstance().getHeight()
					/Workdata::getInstance().getWidth()));
	Settings::getInstance().setValue("map/width",ewidth.text());
	Settings::getInstance().setValue("map/height",eheight.text());
}

void startseite::heightChanged(){
	if(!Workdata::isInitalized())return;
	ewidth.setText(QString::number(eheight.text().toInt()*Workdata::getInstance().getWidth()
					/Workdata::getInstance().getHeight()));

	Settings::getInstance().setValue("map/width",ewidth.text());
	Settings::getInstance().setValue("map/height",eheight.text());
}

void startseite::loadFile(){
	Messaging::message(trUtf8("Datei wird geladen: ")+efilename.text());
	if(!Workdata::init(efilename.text())){
		Messaging::error(trUtf8("Datei konnte nicht geladen werden: ")+efilename.text());
		return;
	}
	Messaging::message(trUtf8("Datei geladen: ")+efilename.text());
	widthChanged();
}

void startseite::mogrifyChanged(){
	Settings::getInstance().setMogrify(emogrify.text());
}
void startseite::convertChanged(){
	Settings::getInstance().setConvert(econvert.text());
}
void startseite::compositeChanged(){
	Settings::getInstance().setComposite(ecomposite.text());
}

QString startseite::information(){
	return trUtf8("Bitte zuerst Pfade für die ImageMagick-Programme convert und mogrify eingeben.\n\
Nach Auswahl des Bildpfades kann die Größe des Spielfeldes eingegeben werden.\n\n\
Das Bild wird anschließend auf 3 Farben reduziert, wobei die dunkelsten Flächen überschwemmt, und \
die hellsten Stellen trocken werden.\n\nVorsicht: Die ausgewählten Voreinstellungen werden nicht \
automatisch überschrieben. Sie müssen explizit auf dem Tab \"Fertigstellen\" gespeichert werden.");
}
