#include "workdata.h"

Workdata::Workdata()
{

}

bool Workdata::init(QString fileName){

	QImage img(fileName);
	if(img.isNull()) return false;
	delete instance;
	instance=new Workdata();
	instance->files["original"]=getTempFileName()+".png";
	Messaging::setProgress(0.5);
	if(!convert("-resize 500x500",fileName,instance->files["original"])){
		delete instance;
		instance=NULL;
		return false;
	}
	img.load(instance->files["original"]);
	instance->imageSize=img.size();
	Messaging::setProgress(1);
	return true;
}

bool Workdata::execute(QString command){
	QProcess p;
	p.start(command);
	if(!p.state()){
		Messaging::error(QObject::trUtf8("Befehl fehlgeschlagen: ")+command);
		return false;
	}
	p.waitForFinished(-1);
	if(p.exitStatus()!=QProcess::NormalExit){
		Messaging::error(QObject::trUtf8("Befehl fehlgeschlagen: ")+command);
		return false;
	}
	return true;
}

void Workdata::setFilename(QString key, QString file){
	files[key]=file;
}

bool Workdata::convert(QString command, QString originalFile, QString outputFile){
	if(!originalFile.isEmpty()&&!originalFile.startsWith("\""))originalFile="\""+originalFile+"\"";
	if(!outputFile.isEmpty()&&!outputFile.startsWith("\""))outputFile="\""+outputFile+"\"";
	return execute(Settings::getInstance().getConvert()+" "+command+" "+originalFile+" "+
				   outputFile);
}

bool Workdata::mogrify(QString command, QString originalFile){
	if(!originalFile.isEmpty()&&!originalFile.startsWith("\""))originalFile="\""+originalFile+"\"";
	return execute(Settings::getInstance().getMogrify()+" "+command+" "+originalFile);
}

bool Workdata::composite(QString file1,QString file2, QString mask, QString out){
	if(!file1.isEmpty()&&!file1.startsWith("\""))file1="\""+file1+"\"";
	if(!file2.isEmpty()&&!file2.startsWith("\""))file2="\""+file2+"\"";
	if(!mask.isEmpty()&&!mask.startsWith("\""))mask="\""+mask+"\"";
	if(!out.isEmpty()&&!out.startsWith("\""))out="\""+out+"\"";
	return execute(Settings::getInstance().getComposite()+" "+file1+" "+file2+" "+mask+" "+out);
}

QString Workdata::getFilename(QString key){
	return files[key];
}

Workdata& Workdata::getInstance(){
	if(!instance)throw "Not initialized!";
	return *instance;
}

QString Workdata::getTempFileName(){
	QTemporaryFile f;
	f.setAutoRemove(false);
	f.open();
	f.close();
	return f.fileName();
}

bool Workdata::isInitalized(){
	return instance;
}

Workdata* Workdata::instance=NULL;

Workdata::~Workdata(){
	foreach(QString file, files){
		QFile::remove(file);
	}
}

int Workdata::getHeight(){
	return imageSize.height();
}

int Workdata::getWidth(){
	return imageSize.width();
}
