#ifndef FIELD_H
#define FIELD_H

#include <QString>
#include <QTemporaryFile>
#include <QImage>
#include <QProcess>
#include <QMessageBox>
#include <QMap>
#include "settings.h"
#include "messaging.h"

//! Enthält die meisten (globalen) temporären Arbeitsdaten des Programms
class Workdata
{
	public:
		//! Destruktor
		~Workdata();
		/*! Initialisiert die Arbeitsdaten mit der Originaldatei
		 * \param filename Original-Bilddatei
		 */
		static bool init(QString filename);
		//! Gibt zurück, ob die Arbeitsdaten initialisiert sind
		static bool isInitalized();
		/*! Gibt die aktuellen Arbeitsdaten zurück
		 * \throws Exception, falls nicht initialisiert
		 * \see init
		 */
		static Workdata& getInstance();
		//! Gibt Dateinamen entsprechend des Keys zurück
		QString getFilename(QString key);
		//! Setzt Dateinamen mit benutzerdefiniertem Key
		void setFilename(QString key, QString file);
		//! Höhe des Originalbilds
		int getHeight();
		//! Breite des Originalbilds
		int getWidth();
		//! Erzeugt neue temporäre Datei
		static QString getTempFileName();

		//! Führt ein Systemkommando aus
		static bool execute(QString command);
		/*! Führt ein ImageMagick-Convert aus
		 * Die Dateinamen können auch im Kommando enthalten sein.
		 */
		static bool convert(QString command,QString originalFile=QString(), QString outputFile=QString());
		/*! Führt ein ImageMagick-Mogrify aus
		 * Die Dateinamen können auch im Kommando enthalten sein.
		 */
		static bool mogrify(QString command,QString originalFile=QString());
		//! Führt ein ImageMagick-Composite aus
		static bool composite(QString file1,QString file2,QString mask, QString out);
	private:
		static Workdata* instance;

		Workdata();

		QSize imageSize;

		QMap<QString,QString> files;

};

#endif // FIELD_H
